"""logs

Revision ID: 7a3913a3bb4c
Revises: 4b751cb703e2
Create Date: 2025-05-15 14:14:22.223452

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '7a3913a3bb4c'
down_revision = '4b751cb703e2'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        'server_logs',
        sa.Column('deployment_logs', sa.String(), server_default='', nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('server_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['server_id'], ['server.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('server_id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('server_logs')
    # ### end Alembic commands ###
