"""creator_id

Revision ID: 735603f5e4d5
Revises: 1c6edbdb4642
Create Date: 2025-05-12 14:04:17.438611

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '735603f5e4d5'
down_revision = '1c6edbdb4642'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('server', sa.Column('creator_id', sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('server', 'creator_id')
    # ### end Alembic commands ###
