"""base_image

Revision ID: f2298e6acd39
Revises: 7e8c31ddb2db
Create Date: 2025-05-28 06:32:44.781544

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'f2298e6acd39'
down_revision = 'aa9a14a698c5'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('server', sa.Column('base_image', sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('server', 'base_image')
    # ### end Alembic commands ###
