"""host_type

Revision ID: 49e1ab420276
Revises: e25328dbfe80
Create Date: 2025-04-25 11:58:17.768233

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '49e1ab420276'
down_revision = 'e25328dbfe80'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('server', sa.Column('host_type', sa.String(), nullable=True))
    op.execute("UPDATE server SET host_type = 'internal'")
    op.alter_column("server", "host_type", nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('server', 'host_type')
    # ### end Alembic commands ###
