"""build_instructions

Revision ID: 4b5c0d56e1ca
Revises: f2298e6acd39
Create Date: 2025-05-30 12:31:08.515098

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '4b5c0d56e1ca'
down_revision = 'f2298e6acd39'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('server', sa.Column('build_instructions', sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('server', 'build_instructions')
    # ### end Alembic commands ###
